import sqlite
import re

# regular expressions 
escapeQuotesA = re.compile("\"")
escapeQuotesB = re.compile("\'")

def escapeQuotes(string):
    """ Replaces quote marks in a string with Unicode escape values """
    string = escapeQuotesA.sub("\\u0022", string)
    return escapeQuotesB.sub("\\u0027", string)

class TripleStore:
    """ Wrapper around a SQL-based RDF store"""
    
    def __init__(self, storeName):
        self.storeName = storeName
        self.tripleCount = 0
        self.sqlStatus = "ok"

    def connect(self):
        """ Connects to the SQLite database """
        self.connection = sqlite.connect(self.storeName, encoding="utf-8")
        
    def commit(self):
        """ Commits a transaction to the database """
        self.connection.commit()

    def getSQLStatus(self):
        """ Returns the status of the last interaction with the database """
        return self.sqlStatus

    def executeSQLFile(self, filename):
        """ Runs the SQL contained in a file """
        f = open(filename, "r")
        sql = f.read()
        f.close()
        
        self.connect()
        cursor = self.connection.cursor()
        self.sqlStatus = "ok"
        try:
            cursor.execute(sql)
        except Exception, (strerror):
            self.sqlStatus = "Error:%s" % (strerror)
        self.connection.commit()
        self.connection.close()

    def addStatement(self, subject, property, object, isLiteral):
        """ adds the statement to the triplestore """

        # escape quote marks to avoid conflict when inserting in SQL statements
        subject = escapeQuotes(subject)
        property = escapeQuotes(property)
        object = escapeQuotes(object)
        
        cursor = self.connection.cursor()
        insert = "INSERT INTO triples \
                    (subject, property, object, has_literal_object) \
                   VALUES \
                   (\'" + subject \
                   + "\',\'" + property \
                   + "\',\'" + object \
                   + "\',\""+str(isLiteral)+"\")"
        cursor.execute(insert) 
        
    def triple(self, subject, property, object):
        """ Callback listener (sink) for pyrple RDF/XML parser """
        self.tripleCount = self.tripleCount + 1
     #   print self.tripleCount

    # get the required strings from the triple data
        try:
            subjectString = subject.val
        except AttributeError:
          #  print "erre"
            subjectString = subject
        try:
            propertyString = property.val
        except AttributeError:
            propertyString = property

    # check if the object is a literal
        if object[0] == "\"":
            isLiteral = True
            objectString = object[1:-1]
        else:
            isLiteral = False
            try:
                objectString = object.val
            except AttributeError: # is a bnode
                objectString = object
 #       subjectString = subjectString.strip("<")
  #      subjectString = subjectString.strip(">")
   #     propertyString = propertyString.strip("<")
    #    propertyString = propertyString.strip(">")
     #   objectString = objectString.strip("<")
      #  objectString = objectString.strip(">")
        
        self.addStatement(subjectString, propertyString, objectString, isLiteral)

        
### RSS -oriented
    def getChannelURIs(self):
        """ Returns all URIs of type rss:channel """
        cursor = self.connection.cursor()
        select = "SELECT * FROM channel_uris"
        try:
            cursor.execute(select)
        except Exception, (strerror):
            self.sqlStatus = "Error:%s" % (strerror)
        self.connection.commit()
        # print self.sqlStatus
        results = cursor.fetchall()
        return results

     def getRecentItems(self, count):
        """ Returns the most recent RSS items """
        cursor = self.connection.cursor()
        select = "\
            SELECT * FROM items \
            LIMIT " + str(count)
        try:
            cursor.execute(select)
        except Exception, (strerror):
            self.sqlStatus = "Error:%s" % (strerror)
        self.connection.commit()
        # print self.sqlStatus
        results = cursor.fetchall()
        return results
    
### Utility
    def setUniqueProperty(self, subject, property, object, isLiteral):
        """ removes triples with matching subject and property 
            then adds a new triple (with object) """
        cursor = self.connection.cursor()
        subject = escapeQuotes(subject)
        property = escapeQuotes(property)
        self.sqlStatus = "ok"
        delete = "DELETE FROM triples \
            WHERE triples.subject == \'" + subject \
                   + "\' AND triples.property == \'" + property \
                   + "\' AND triples.has_literal_object == \""+str(isLiteral) \
                   +"\""
        #print delete
        try:
            cursor.execute(delete)
        except Exception, (strerror):
            self.sqlStatus = "Error:%s" % (strerror)
        self.connection.commit()
        self.addStatement(subject, property, object, isLiteral)
        print self.sqlStatus
    def setChannelStatus(self, channel, status):
        """ Sets the status flag of RSS channels """
        channel = escapeQuotes(channel)
        cursor = self.connection.cursor()
        self.setUniqueProperty(channel,
                "http://purl.org/stuff/status",
                status,
                "True")

    def getChannelStatus(self, uri):
        """ Returns the status flag of RSS channels """
        uri = escapeQuotes(uri)
        cursor = self.connection.cursor()
        select = "SELECT triples.object FROM triples \
        WHERE \
        triples.subject == \""+uri+"\" \
        AND \
        triples.property == \"http://purl.org/stuff/status\""
        self.connection.commit()
        return cursor.fetchone()        
        
        try:
            cursor.execute(select)
        except Exception, (strerror):
            self.sqlStatus = "Error:%s" % (strerror)
            
        self.connection.commit()
        results = cursor.fetchall()
        return results


    

    
                
   #     update = "UPDATE triples \
    #    SET object = \""+status+ "\" \
     #   WHERE \
      #  property == \"http://purl.org/stuff/status\" \
       # AND \
       # subject == \""+channel+"\""
        # print update
       # self.sqlStatus = "ok"
        #try:
         #   cursor.execute(update)
       # except Exception, (strerror):
        #    self.sqlStatus = "Error:%s" % (strerror)
       # self.connection.commit()
       # print self.sqlStatus        



    
        
            



        
        

          
