CREATE TABLE triples ( 
                subject TEXT, 
                property TEXT,
                object TEXT,
                has_literal_object BOOL);

CREATE INDEX spo ON triples (subject, property, object);            

CREATE VIEW channel_uris AS SELECT
Channels.subject
FROM
triples Channels
WHERE
Channels.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Channels.object ==  "http://purl.org/rss/1.0/channel";


CREATE VIEW items AS 
SELECT DISTINCT
Items.subject,
Titles.object,
Descriptions.object,
Dates.object
FROM
triples Items,
triples Titles,
triples Descriptions,
triples Dates
WHERE
Items.has_literal_object == "False"
AND
Items.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Items.object == "http://purl.org/rss/1.0/item"

AND
Titles.subject ==  Items.subject
AND
Titles.property == "http://purl.org/rss/1.0/title"

AND
Descriptions.subject ==  Items.subject
AND
Descriptions.property == "http://purl.org/rss/1.0/description"

AND
Dates.subject ==  Items.subject
AND
(Dates.property == "http://purl.org/dc/elements/1.1/date"
OR
Dates.property == "http://purl.org/dc/terms/created"
OR
Dates.property == "http://purl.org/dc/terms/modified")

ORDER BY Dates.object DESC;

CREATE VIEW blogroll AS
SELECT
Names.object,
Documents.subject,
Titles.object,
seeAlsos.object
FROM
triples Agents,
triples Names,
triples weblogs,
triples Documents,
triples Titles,
triples seeAlsos,
triples Channels
WHERE
Agents.has_literal_object == "False"
AND
Agents.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Agents.object == "http://xmlns.com/foaf/0.1/Agent"

AND
Names.subject ==  Agents.subject
AND
Names.property == "http://xmlns.com/foaf/0.1/name"
AND
weblogs.subject ==  Agents.subject
AND
weblogs.property == "http://xmlns.com/foaf/0.1/weblog"
AND
weblogs.object == Documents.subject
AND

Documents.has_literal_object == "False"
AND
Documents.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Documents.object == "http://xmlns.com/foaf/0.1/Document"

AND
Titles.subject ==  Documents.subject
AND
Titles.property == "http://purl.org/dc/elements/1.1/title"

AND
seeAlsos.subject ==  Documents.subject
AND
seeAlsos.property == "http://www.w3.org/2000/01/rdf-schema#seeAlso"

AND
Channels.subject ==  seeAlsos.object
AND
Channels.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Channels.object ==  "http://purl.org/rss/1.0/channel";

CREATE VIEW jobs AS SELECT
JobTitles.object,
JobSalaries.object
FROM
triples Jobs,
triples JobTitles,
triples JobSalaries
WHERE
Jobs.property == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"
AND
Jobs.object == "http://xmlns.com/wordnet/1.6/Job"
AND
JobTitles.property == "http://ilrt.org/discovery/2000/11/rss-query/jobvocab.rdf#title"
AND
JobTitles.subject == Jobs.subject
AND
JobSalaries.property == "http://ilrt.org/discovery/2000/11/rss-query/jobvocab.rdf#salary"
AND
JobSalaries.subject == Jobs.subject;

CREATE VIEW find_dupes AS
SELECT * FROM triples
GROUP BY
subject,
property,
object,
has_literal_object
HAVING COUNT(*) > 1;

