using System;
using System.Collections;
using System.Xml;

namespace  Rooster
{
	
	public class DomFeed: FeedBase
	{
		string RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
		string RSS_NS = "http://purl.org/rss/1.0/";
		string ATOM_NS = "http://purl.org/atom/ns#";
		
		XmlDocument	doc;
		XmlNamespaceManager nsManager;
		
		private  ArrayList items = null;
		
		
		public DomFeed(IFeedStore store, string name, string uri):
			base(store, name, uri){
			doc	=	new XmlDocument();
			initNSManager();
		}
		
		private void initNSManager(){
			nsManager = new XmlNamespaceManager(doc.NameTable);
			nsManager.AddNamespace("rss", RSS_NS);
			nsManager.AddNamespace("rdf", RDF_NS);
			nsManager.AddNamespace("atom", ATOM_NS);
		}
		
		public override void loadData(){
			string title = getFeedTitle();
			try{
				OnFeedEvent("Loading: "+URI);
				doc.Load(URI);
				readItems();
				OnFeedEvent("Loaded: "+URI);
			}catch(Exception exception){
				OnFeedEvent("Error: "+exception);
				return;
			}
		}
		
		
		public override ArrayList Items {
			get{ return items; }
			set{ this.items = value; }
		}
		
		// /rdf:RDF//rss:item/rss:title/text()
		private void readItems(){
			items = new ArrayList();
			XmlElement root = doc.DocumentElement;
			XmlNodeList itemNodes = root.SelectNodes("//item|rss:item|atom:entry", nsManager);
			foreach(XmlNode itemNode in itemNodes){
				string uri = "";
				XmlNode linkNode = itemNode.SelectSingleNode("link");
				if(linkNode == null){
					linkNode = itemNode.SelectSingleNode("@rdf:about", nsManager);
				}
				if(linkNode == null){
					linkNode = itemNode.SelectSingleNode("guid");
				}
				if(linkNode != null){
					uri = linkNode.InnerText;
				}
				string title = "";
				XmlNode titleNode = itemNode.SelectSingleNode("title");
				if(titleNode != null){
					title = titleNode.InnerText;
				}
				string content = "";
				XmlNode contentNode = itemNode.SelectSingleNode("description");
				if(contentNode != null){
					content = contentNode.InnerText;
				}
				Item	item = new Item(uri,title,content);
				items.Add(item);
			}
		}
		
		
		public string getFeedTitle(){
			XmlElement root = doc.DocumentElement;
			if(root == null){
				return null;
			}
			XmlNode title = root.SelectSingleNode("/rss/channel/title", nsManager);
			if(title != null){
				return title.InnerText;
			}
			title = root.SelectSingleNode("/rdf:RDF/rss:channel/rss:title",  nsManager);
			if(title != null){
				return title.InnerText;
			}
			title = root.SelectSingleNode("/atom:feed/atom:title", nsManager);
			if(title != null){
				return title.InnerText;
			}
			return "untitled";
		}
	}
}

