
using System;
using System.Windows.Forms;
using System.Timers;
using System.Collections;

namespace  Rooster
{
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TextBox contentTextBox;
		private System.Windows.Forms.RichTextBox richTextBox1;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.Splitter horizontalSplitter;
		private System.Windows.Forms.Panel feedsPanel;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.Panel contentPanel;
		private System.Windows.Forms.ListBox titlesListBox;
		private System.Windows.Forms.Panel titlesPanel;
		private System.Windows.Forms.Splitter verticalSplitter;
		private System.Windows.Forms.TreeView feedTreeView;
		private IFeedStore store;
		
		public delegate void FeedEventHandler(object sender, FeedEventArgs e);
		
		public void OnFeedEvent(object sender,
		                        FeedEventArgs e) {
			
			this.statusBar.Text = e.Status;
			this.statusBar.Refresh();
		}
		
		
		public MainForm(){
			
			InitializeComponent();
			CreateMainMenu();
			
			this.feedTreeView.AfterSelect += new TreeViewEventHandler(this.treeSelect);
			this.titlesListBox.SelectedIndexChanged += new EventHandler(this.listSelect);
		}
		
		public void treeSelect(object sender, TreeViewEventArgs e){
			
			IFeed selectedFeed = store.selectNamedFeed(e.Node.Text);
			if(selectedFeed == null){
				return;
			}
			selectedFeed.loadData();
		// selectedFeed.update();

			placeTitles(selectedFeed, titlesListBox);
			contentTextBox.Clear();
			this.statusBar.Text = e.Node.Text+" : "+selectedFeed.URI; ///
			this.statusBar.Refresh();
		
		}
		
		public void placeTitles(IFeed feed, ListBox titlesListBox){
			titlesListBox.Items.Clear();
			if(feed == null){
				return;
			}
			foreach(Item item in feed.Items){
				titlesListBox.Items.Add(item);
			}
			titlesListBox.Refresh();
		}
		
		public void listSelect(object sender, EventArgs e){
			contentTextBox.Clear();
			object selectedItem = ((ListBox)sender).SelectedItem;
			string content =	((Item)selectedItem).Content;
			contentTextBox.AppendText(content);
		}
		
		
		
		[STAThread]
		public static void Main(string[] args)
		{
			Application.Run(new MainForm());
		}
		
		
		public void CreateMainMenu()
		{
			MenuItem fileMenuItem = new MenuItem();
			fileMenuItem.Text = "File";
			mainMenu.MenuItems.Add(fileMenuItem);
			
			MenuItem importMenuItem = new MenuItem();
			importMenuItem.Text = "Import";
			fileMenuItem.MenuItems.Add(importMenuItem);
			importMenuItem.Click += new System.EventHandler(this.importMenuItemClick);
		}
		
		void importMenuItemClick(object sender, System.EventArgs e)
		{
			OpenFileDialog openDialog = new OpenFileDialog();
			openDialog.Filter = "feedlist files (*.rdf)|*.opml|All files (*.*)|*.*" ;
			openDialog.FilterIndex = 2 ;
			openDialog.RestoreDirectory = true ;
			openDialog.InitialDirectory = ".";
			
			if(openDialog.ShowDialog() == DialogResult.OK)
			{
				if(openDialog.FileName.EndsWith(".rdf")){
					store = new CarpFeedStore();
				}else{
					store = new DomFeedStore();
				}
				store.loadChannelsList(openDialog.FileName);
				initFeedView();
			}
			
		}
		
		public void initFeedView(){
			feedTreeView.Nodes.Clear();
			IDictionaryEnumerator enumerator = store.GetFeedEnumerator();
			
			while(enumerator.MoveNext()){
				string name = (string)enumerator.Key;
				IFeed feed = (IFeed)enumerator.Value;
				
				feedTreeView.Nodes.Add(new TreeNode(name));
				((FeedEventTrigger)feed).feedEventHandler += new FeedEventHandler(OnFeedEvent);
			}
			
		}
		
		
		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.feedTreeView = new System.Windows.Forms.TreeView();
			this.verticalSplitter = new System.Windows.Forms.Splitter();
			this.titlesPanel = new System.Windows.Forms.Panel();
			this.titlesListBox = new System.Windows.Forms.ListBox();
			this.contentPanel = new System.Windows.Forms.Panel();
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.feedsPanel = new System.Windows.Forms.Panel();
			this.horizontalSplitter = new System.Windows.Forms.Splitter();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.richTextBox1 = new System.Windows.Forms.RichTextBox();
			this.contentTextBox = new System.Windows.Forms.TextBox();
			this.titlesPanel.SuspendLayout();
			this.contentPanel.SuspendLayout();
			this.feedsPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// feedTreeView
			// 
			this.feedTreeView.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
			                                                                 | System.Windows.Forms.AnchorStyles.Left)));
			this.feedTreeView.ImageIndex = -1;
			this.feedTreeView.Location = new System.Drawing.Point(0, 0);
			this.feedTreeView.Name = "feedTreeView";
			this.feedTreeView.SelectedImageIndex = -1;
			this.feedTreeView.Size = new System.Drawing.Size(200, 512);
			this.feedTreeView.TabIndex = 1;
			// 
			// verticalSplitter
			// 
			this.verticalSplitter.Location = new System.Drawing.Point(200, 0);
			this.verticalSplitter.Name = "verticalSplitter";
			this.verticalSplitter.Size = new System.Drawing.Size(3, 511);
			this.verticalSplitter.TabIndex = 1;
			this.verticalSplitter.TabStop = false;
			// 
			// titlesPanel
			// 
			this.titlesPanel.Controls.Add(this.titlesListBox);
			this.titlesPanel.Dock = System.Windows.Forms.DockStyle.Top;
			this.titlesPanel.Location = new System.Drawing.Point(203, 0);
			this.titlesPanel.Name = "titlesPanel";
			this.titlesPanel.Size = new System.Drawing.Size(597, 100);
			this.titlesPanel.TabIndex = 2;
			// 
			// titlesListBox
			// 
			this.titlesListBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.titlesListBox.Location = new System.Drawing.Point(0, 0);
			this.titlesListBox.Name = "titlesListBox";
			this.titlesListBox.Size = new System.Drawing.Size(597, 100);
			this.titlesListBox.TabIndex = 0;
			// 
			// contentPanel
			// 
			this.contentPanel.Controls.Add(this.contentTextBox);
			this.contentPanel.Dock = System.Windows.Forms.DockStyle.Fill;
			this.contentPanel.Location = new System.Drawing.Point(203, 103);
			this.contentPanel.Name = "contentPanel";
			this.contentPanel.Size = new System.Drawing.Size(597, 408);
			this.contentPanel.TabIndex = 4;
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 511);
			this.statusBar.Name = "statusBar";
			this.statusBar.Size = new System.Drawing.Size(800, 22);
			this.statusBar.TabIndex = 2;
			this.statusBar.Text = "OK";
			// 
			// feedsPanel
			// 
			this.feedsPanel.Controls.Add(this.feedTreeView);
			this.feedsPanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.feedsPanel.Location = new System.Drawing.Point(0, 0);
			this.feedsPanel.Name = "feedsPanel";
			this.feedsPanel.Size = new System.Drawing.Size(200, 511);
			this.feedsPanel.TabIndex = 0;
			// 
			// horizontalSplitter
			// 
			this.horizontalSplitter.Dock = System.Windows.Forms.DockStyle.Top;
			this.horizontalSplitter.Location = new System.Drawing.Point(203, 100);
			this.horizontalSplitter.Name = "horizontalSplitter";
			this.horizontalSplitter.Size = new System.Drawing.Size(597, 3);
			this.horizontalSplitter.TabIndex = 3;
			this.horizontalSplitter.TabStop = false;
			// 
			// richTextBox1
			// 
			this.richTextBox1.AutoSize = true;
			this.richTextBox1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.richTextBox1.Location = new System.Drawing.Point(0, 0);
			this.richTextBox1.Name = "richTextBox1";
			this.richTextBox1.Size = new System.Drawing.Size(792, 443);
			this.richTextBox1.TabIndex = 0;
			this.richTextBox1.Text = "";
			// 
			// contentTextBox
			// 
			this.contentTextBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.contentTextBox.Location = new System.Drawing.Point(0, 0);
			this.contentTextBox.Multiline = true;
			this.contentTextBox.Name = "contentTextBox";
			this.contentTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.contentTextBox.Size = new System.Drawing.Size(597, 408);
			this.contentTextBox.TabIndex = 0;
			this.contentTextBox.Text = "";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(800, 533);
			this.Controls.Add(this.contentPanel);
			this.Controls.Add(this.horizontalSplitter);
			this.Controls.Add(this.titlesPanel);
			this.Controls.Add(this.verticalSplitter);
			this.Controls.Add(this.feedsPanel);
			this.Controls.Add(this.statusBar);
			this.Location = new System.Drawing.Point(4, 22);
			this.Menu = this.mainMenu;
			this.Name = "MainForm";
			this.Text = "Rooster";
			this.titlesPanel.ResumeLayout(false);
			this.contentPanel.ResumeLayout(false);
			this.feedsPanel.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion
		
		
	}
}


