package org.imp;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

public class Imp implements ActionListener, AudioTool {

    private JFrame window;

    private AudioTool audioTool;

    private JProgressBar meterBar;

    private JButton recordButton;

    private JButton stopButton;

    private JButton playButton;
    
    private JButton publishButton;

    private int level = 0;

    private boolean showLevel;

    private LevelUpdate levelUpdate;
    
    public static Settings settings;

    private int length = -1;

    public static void main(String[] args) {
        settings = new Settings();
        new Imp();
    }

    public Imp() {
        JFrame window = createWindow();

        Container contentPane = window.getContentPane();
        contentPane.setLayout(new BorderLayout());

        JPanel buttonPanel = createButtonPanel();
        contentPane.add(buttonPanel, BorderLayout.SOUTH);

        meterBar = createMeterBar();
        contentPane.add(meterBar, BorderLayout.CENTER);

        enableButtons(true, false, false, false);
   //     enableButtons(true, false, true, true); //handy for testing 
        window.setVisible(true);
    }

    private JFrame createWindow() {
        window = new JFrame("Imp");
        centerWindow(window,300,80);
        return window;
    }
    
    public static void centerWindow(Window window, int width, int height){
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension size = new Dimension(width, height);
        window.setSize(size);
        window.setLocation((screenSize.width - size.width) / 2, 
                (screenSize.height - size.height) / 2);
    }

    private JPanel createButtonPanel() {
        JPanel buttonPanel = new JPanel();

        recordButton = new JButton("Record");
        recordButton.addActionListener(this);
        buttonPanel.add(recordButton);

        stopButton = new JButton("Stop");
        stopButton.addActionListener(this);
        buttonPanel.add(stopButton);

        playButton = new JButton("Play");
        playButton.addActionListener(this);
        buttonPanel.add(playButton);
        
        publishButton = new JButton("Publish");
        publishButton.addActionListener(this);
        buttonPanel.add(publishButton);
        return buttonPanel;
    }

    private JProgressBar createMeterBar() {
        meterBar = new MeterBar(this, 0, 128);
        meterBar.setBorderPainted(true);
        meterBar.setForeground(Color.green);
        return meterBar;
    }

    private void enableButtons(boolean start, boolean stop, boolean play, boolean publish) {
        recordButton.setEnabled(start);
        stopButton.setEnabled(stop);
        playButton.setEnabled(play);
        publishButton.setEnabled(publish);
    }

    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (command.equals("Record")) {
            audioTool = new Recorder();
            audioTool.startActivity();
            startMeter();
            enableButtons(false, true, false, false);
        }

        else if (command.equals("Stop")) {
            stopActivity();
       
        } else if (command.equals("Play")) {
            //    captureMode = false;
            audioTool = new Player();
            levelUpdate.setMetered(audioTool);
            audioTool.startActivity();
            enableButtons(false, true, false, false);
            startMeter();
        } else if (command.equals("Publish")) {
            EnclosureEntry entry = new EnclosureEntry();
       
            entry.setLength(Integer.toString(getLength()));
            (new Publisher()).publish(entry);
        
        }
    }

    public void stopActivity() {
        showLevel = false;
        audioTool.stopActivity();
        length = audioTool.getLength();
        enableButtons(true, false, true, true);
    }

    public void startMeter() {
        levelUpdate = new LevelUpdate();
        levelUpdate.setMetered(audioTool);
        levelUpdate.start();
    }
    
   
    public int getLength() {
        return length;
    }

    class LevelUpdate extends Thread {
        float oldlevel = 0;

        AudioTool active;

        public void setMetered(AudioTool active) {
            this.active = active;
        }

        public void run() {
            level = 0;
            meterBar.setBackground(Settings.meterBackgroundColor);
            meterBar.repaint();
            while (active.getLevelVisible()) {
                //  level = captureMode ? rec.getLevel() : play.getLevel();
                level = active.getLevel();

                //     if (!captureMode && play.isEnd()) {
                //       showLevel = false;
                // }
                if (level != oldlevel) {
                    meterBar.repaint();
                    oldlevel = level;
                }
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ie) {
                    System.out.println(ie);
                }
            }
            enableButtons(true, false, true, false);
            meterBar.setBackground(SystemColor.control);
            meterBar.repaint();
            level = 0;
        }
    }

    public int getLevel() {
        return level;
    }

    public boolean getLevelVisible() {
        return audioTool.getLevelVisible();
    }

    public void startActivity() {
        // not used
    }
}