/*
 * Record.java
 */

package org.imp;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

public class Recorder extends Thread  implements AudioTool {

    private TargetDataLine targetDataLine;

    private AudioInputStream audioInputStream;

    private ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();

    private int peak; 
    private int length = -1;

    private File file;
 
    private boolean recording;

    public Recorder() {
        file = new File(Settings.audioFilename);
        if (file.exists()) {
            file.delete();
        }
    }

    public TargetDataLine getTargetDataLine() {
        DataLine.Info info = new DataLine.Info(TargetDataLine.class,
                Settings.format);
        TargetDataLine tdline = null;
        try {
            tdline = (TargetDataLine) AudioSystem.getLine(info);
            tdline.open(Settings.format, Settings.bufferSize);
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        }
        return tdline;
    }

    public void run() {
        byte[] buffer = new byte[Settings.bufferSize];
        int nBufferFrames = Settings.bufferSize / Settings.frameSize;

        recording = true;
        while (recording) {
            int nFramesRead = targetDataLine.read(buffer, 0, nBufferFrames);
            peak = 0;
            for (int i = 1; i < nFramesRead - 1; i += Settings.frameSize * 2) {
                peak = buffer[i] > peak ? buffer[i] : peak;
            }
            if (peak > Settings.threshold) {
                byteArrayOutputStream.write(buffer, 0, nFramesRead);
            }
        }
    }

    public void startActivity() {
        targetDataLine = getTargetDataLine();
        targetDataLine.start();
        start();
    }

    public void stopActivity() {
        targetDataLine.drain();
        targetDataLine.stop();
        targetDataLine.close();
        recording = false;
        try {
            byteArrayOutputStream.close();

            byte[] byteData = byteArrayOutputStream.toByteArray();
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
                    byteData);
            audioInputStream = new AudioInputStream(byteArrayInputStream,
                    Settings.format, byteData.length / Settings.frameSize);

           length = writeAudioFile(audioInputStream, file); //
  
            audioInputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public int getLevel() {
        return peak;
    }

    public static int writeAudioFile(AudioInputStream inputStream, File outFile)
            throws Exception {

        AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(
                Settings.MPEG1L3, inputStream);
        if (audioInputStream == null) {
            throw new Exception("Conversion not supported.");
        }
        int writtenBytes = AudioSystem.write(audioInputStream, Settings.MP3,
                outFile);
        audioInputStream.close();
        Settings.showProperties(System.getProperties());
        return writtenBytes;
    }

    public boolean getLevelVisible() {
        return true;
    }

    public int getLength() {
        return length;
    }
}

