package org.imp;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Properties;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioFormat.Encoding;

import org.tritonus.share.sampled.AudioFileTypes;
import org.tritonus.share.sampled.Encodings;

public class Settings {

    public static String SETTINGS_FILE = "settings.txt";

    public static Encoding encoding = Encoding.PCM_SIGNED;

    public static final AudioFormat.Encoding MPEG1L3 = Encodings
            .getEncoding("MPEG1L3");

    public static final AudioFileFormat.Type MP3 = AudioFileTypes.getType(
            "MP3", "mp3");

    public static final Color meterBackgroundColor = Color.black;

    private Properties properties;

    public static float sampleRate;

    public static int resolution;

    public static int nChannels;

    public static int frameSize;

    public static float frameRate;

    public static boolean bigEndian;

    public static AudioFormat format;

    public static int bufferSize;

    public static int threshold;

    public static String audioFilename;

    public static String httpHost;

    public static String path;

    public static String rssFilename;

    public static String user;

    public static String pass;

    public static int maxItems;

    public static String ftpHost;

    public Settings() {
        properties = new Properties();
        try {
            properties.load(new FileInputStream(SETTINGS_FILE));
        } catch (Exception e) {
            e.printStackTrace();
        }

        sampleRate = getFloat("sampleRate");
        resolution = getInt("resolution");
        nChannels = getInt("nChannels");
        frameSize = getInt("frameSize");
        frameRate = getFloat("frameRate");
        bigEndian = getBoolean("bigEndian");

        format = new AudioFormat(encoding, sampleRate, resolution, nChannels,
                frameSize, frameRate, bigEndian);

        bufferSize = getInt("bufferSize");
        threshold = getInt("threshold");
        audioFilename = getString("audioFilename");

        httpHost = getString("httpHost");
        ftpHost = getString("ftpHost");
        path = getString("path");
        rssFilename = getString("rssFilename");
        user = getString("user");
        pass = getString("pass");

        maxItems = getInt("maxItems");
    }

    public int getInt(String name) {
        String stringValue = properties.getProperty(name);
        return Integer.parseInt(stringValue);
    }

    public float getFloat(String name) {
        String stringValue = properties.getProperty(name);
        return Float.parseFloat(stringValue);
    }

    public boolean getBoolean(String name) {
        String stringValue = properties.getProperty(name);
        return Boolean.getBoolean(stringValue);
    }

    public String getString(String name) {
        return properties.getProperty(name);
    }

    public static void showProperties(Properties properties) {
        Enumeration keys = properties.keys();
        while (keys.hasMoreElements()) {
            String key = (String) keys.nextElement();
            String value = (String) properties.getProperty(key);
            if (key.startsWith("tritonus.lame.effective")) {
                System.out.println(key.substring(24) + " = " + value);
            }
        }
    }

    //  methods below only used to bootstrap - can be safely ignored
    public static void main(String[] args) {
        Settings settings = new Settings();
        //
        settings.set("sampleRate", "22050.0F");
        settings.set("resolution", "16");
        settings.set("nChannels", "1");
        settings.set("frameSize", "2");
        settings.set("frameRate", "22050.0F");
        //
        settings.set("bufferSize", "16384");
        settings.set("threshold", "10");
        settings.set("audioFilename", "audio.mp3");
        //
        settings.set("host", "http://semtext.org");
        settings.set("rssFilename", "danny666/audio/audio.xml");
        settings.set("user", "username");
        settings.set("pass", "password");
        settings.save();
    }

    public void set(String name, String value) {
        properties.setProperty(name, value);
    }

    public void save() {
        try {
            properties.save(new FileOutputStream(SETTINGS_FILE),
                    "Imp Properties");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}